#!/usr/bin/python3
# 3/28/2019 Skeetzo
# OnlySnarf.py menu system

### doesn't work:
# upload & backup (requires upload via local added to main script)
# settings menu -> "Incorrect Index"

import random
import os
import shutil
import datetime
import json
import sys
import pathlib
import pkg_resources
from OnlySnarf.settings import SETTINGS as settings
from OnlySnarf import onlysnarf as OnlySnarf
from OnlySnarf import google as Google

###################
##### Globals #####
###################

version = str(pkg_resources.get_distribution("onlysnarf").version)
header = "\n ________         .__          _________                     _____ \n \
\\_____  \\   ____ |  | ___.__./   _____/ ____ _____ ________/ ____\\\n \
 /   |   \\ /    \\|  |<   |  |\\_____  \\ /    \\\\__  \\\\_   _ \\   __\\ \n \
/    |    \\   |  \\  |_\\___  |/        \\   |  \\/ __ \\ |  |\\/| |   \n \
\\_______  /___|  /____/ ____/_______  /___|  (____  \\\\__|  |_|   \n \
        \\/     \\/     \\/            \\/     \\/     \\/              \n"

UPDATED = False
UPDATED_TO = False
INITIALIZED = False

colors = {
        'blue': '\033[94m',
        'header': '\033[48;1;34m',
        'teal': '\033[96m',
        'pink': '\033[95m',
        'green': '\033[92m',
        'yellow': '\033[93m',
        'menu': '\033[48;1;44m'
        }

menuItems = []
actionItems = []
messageItems = []
fileItems = []
locationItems = []
promotionItems = []
settingItems = []

def initialize():
    # print("Initializing Menu")
    global INITIALIZED
    if INITIALIZED:
        # print("Already Initialized, Skipping")
        return
    global settingItems
    # Settings Menu
    settingItems = sorted([
        [ "Backup", settings.BACKUP, ["True","False"],True],
        [ "Delete Google", settings.DELETE_GOOGLE, ["True","False"],False],
        [ "Delete Local", settings.SKIP_DELETE, ["True","False"],False],
        [ "Force Backup", settings.FORCE_BACKUP, ["True","False"],False],
        [ "Force Upload", settings.FORCE_UPLOAD, ["True","False"],False],
        [ "Mount Path", settings.PATH_MOUNT,None,False],
        [ "Google: Root Folder Path", settings.PATH_DRIVE,None,False],
        [ "Users Path", settings.PATH_USERS,None,False],
        [ "Show Window", settings.SHOW_WINDOW, ["True","False"],False],
        [ "Google: Root Folder Name", settings.ROOT_FOLDER,None,False],
        [ "Google: Drive Folders", settings.DRIVE_FOLDERS,None,False],
        [ "Image Limit", settings.IMAGE_UPLOAD_LIMIT,None,True],
        [ "Image Max", settings.IMAGE_UPLOAD_MAX,None,False],
        [ "Tweeting", settings.TWEETING, ["True","False"],True],
        [ "Debug", settings.DEBUG, ["True","False"],False],
        [ "Debug Skip Download", settings.SKIP_DOWNLOAD, ["True","False"],False],    
        [ "Create Missing Google Folders", settings.DRIVE_CREATE_MISSING, ["True","False"],False],    
        [ "Verbose", settings.VERBOSE, ["True","False"],True]    
    ])
    if str(settings.DEBUG) == "True":
        settingItems.append([ "Text", settings.TEXT,None,False])
        settingItems.append([ "Image", settings.IMAGE,None,False])
        settingItems.append([ "Prefer Local", settings.PREFER_LOCAL,["True","False"],True])
    settingItems.insert(0,[ "Back", "main"])

    global menuItems
    # Main Menu
    menuItems = [
        [ "Actions", "action"],
        [ "Settings", "set_settings"],
        [ "Exit", "exit"]
    ]

    global actionItems
    # Actions Menu
    actionItems = sorted([
        [ "Release", "release" ],
        [ "Download", "download" ],
        # [ "Promotion", "promotion" ],
        [ "Message", "message" ],
        [ "Reset", "reset" ]
    ])
    if str(settings.DEBUG) == "True":
        actionItems.append([ "Test", "test"])
        actionItems.append([ "Promotion", "promotion" ])
    actionItems.insert(0,[ "Back", "main"])

    global messageItems
    # Message Menu
    messageItems = sorted([
        [ "All", "all"],
        [ "New", "new"],
        [ "Recent", "recent"],
        [ "User by Username", "user"]
    ])
    messageItems.insert(0,[ "Back", "main"])

    global fileItems
    # File Type Menu
    fileItems = sorted([
        [ "Image", "image"],
        [ "Gallery", "gallery"],
        [ "Performer", "performer"],
        # [ "Scene", "scene"],
        [ "Video", "video"],
    ])
    if str(settings.DEBUG) == "True":
        fileItems.append([ "Scene", "scene"])
    fileItems.insert(0,[ "Back", "main"])

    global locationItems
    # File Location Menu
    locationItems = sorted([
        [ "Local", "local"],
        [ "Google Drive", "google"]
    ])
    locationItems.insert(0,[ "Back", "main"])

    global promotionItems
    if str(settings.DEBUG) == "True":
        promotionItems = sorted([
            [ "Enter Email", "email" ],
            # [ "Select User", "select" ]
        ])
    promotionItems.insert(0,[ "Back", "main"])

    # print("Initialized Menu")
    INITIALIZED = True

#####################
##### Functions #####
#####################

### Action Menu - file type
def action():
    for item in actionItems:
        print(colorize("[" + str(actionItems.index(item)) + "] ", 'teal') + list(item)[0])
    while True:
        choice = input(">> ")
        try:
            if int(choice) < 0 or int(choice) >= len(actionItems): raise ValueError
            if str(actionItems[int(choice)][1]) == "main":
                return main()
            elif str(actionItems[int(choice)][1]) == "reset":
                OnlySnarf.remove_local()
            elif str(actionItems[int(choice)][1]) == "message":
                actionChoice = list(actionItems[int(choice)])[1]
                return finalizeMessage(actionChoice)
            elif str(actionItems[int(choice)][1]) == "promotion":
                actionChoice = list(actionItems[int(choice)])[1]
                return finalizePromotion(actionChoice)
            else:
                actionChoice = list(actionItems[int(choice)])[1]
                return finalizeAction(actionChoice)
        except (ValueError, IndexError):
            print("Error: Incorrect Index")
        except Exception as e:
            settings.maybePrint(e)
            print("Error: Missing Method") 

### Action Menu - finalize
def finalizeAction(actionChoice):
    for item in fileItems:
        print(colorize("[" + str(fileItems.index(item)) + "] ", 'teal') + list(item)[0])
    while True:
        fileChoice = input(">> ")
        try:
            if int(fileChoice) < 0 or int(fileChoice) >= len(fileItems): raise ValueError
            if str(fileItems[int(fileChoice)][1]) == "main":
                return action()
            # Call the matching function
            fileChoice = list(fileItems[int(fileChoice)])[1]
            return performAction(actionChoice, fileChoice)
        except (ValueError, IndexError):
            print("Error: Incorrect Index")
        except Exception as e:
            settings.maybePrint(e)
            print("Error: Missing Method") 

### Action Menu - perform
def performAction(actionChoice, fileChoice):
    try:
        method = getattr(OnlySnarf, str(actionChoice))
        response = method(fileChoice)
        if response:
            if str(actionChoice) == "download":
                for setting in settingItems:
                    if setting[0] == "File Name":
                        setting[1] = response[0]
                    elif setting[0] == "File Path":
                        setting[1] = response[1]
    except (ValueError, IndexError):
        print("Error: Incorrect Index")
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Missing Method") 
    mainMenu()

# Message Menu - finalize
def finalizeMessage(actionChoice):
    for item in messageItems:
        print(colorize("[" + str(messageItems.index(item)) + "] ", 'teal') + list(item)[0])
    while True:
        choice = input(">> ")
        try:
            choice = int(choice)
            if int(choice) < 0 or int(choice) >= len(messageItems): raise ValueError
            if str(messageItems[int(choice)][1]) == "main":
                return action()
            return performMessage(actionChoice, messageItems[int(choice)][1])
        except (ValueError, IndexError):
            print(sys.exc_info()[0])
            print("Error: Incorrect Index")

# Message Menu - perform
def performMessage(actionChoice, messageChoice):
    username = None
    if str(messageChoice) == "user":
        users = displayUsers()
        seeking = True
        while seeking:
            choice = input(">> ")
            try:
                if int(choice) < 0 or int(choice) > len(users): raise ValueError
                if int(choice) == 0:
                    return finalizeMessage(actionChoice)
                username = users[int(choice)-1].username
                seeking = False
            except (ValueError, IndexError):
                print(sys.exc_info()[0])
                print("Error: Incorrect Index")
                return mainMenu()
    images = selectImage(messageChoice)
    # [folder , image_file]
    # print("len: " + str(len(images)))
    while True:
        choice = input(">> ")
        try:
            if int(choice) < 0 or int(choice) > len(images): raise ValueError
            if int(choice) == 0:
                return finalizeMessage(actionChoice)
            try:
                image = images[int(choice)-1]
                message(messageChoice, [image[1],image[0]], username)
                return mainMenu()
            # except (ValueError, IndexError):
                # print("Error: Incorrect Index")
            except Exception as e:
                settings.maybePrint(e)
                print("Error: Missing Method")
        except (ValueError, IndexError):
            print("Error: Incorrect Index")
    # mainMenu()    

# {
#   image: file
#   folder: folder
# }
def message(choice, image=None, username=None):
    message = input("Message: ")
    waiting = True
    while waiting:
        try:
            price = input("Price: ")
            "{:.2f}".format(float(price))
            waiting = False
        except ValueError:
            print("Enter a currency amount!")
    if not image or not image[0] or image[0] == None:
        print("Error: Missing Image")
        return
    file_path = Google.download_file(image[0])
    successful_message = OnlySnarf.message(choice=choice, message=message, image=file_path, price=price, username=username)
    if successful_message and str(choice) != "new":
        Google.move_file(image[0])
    else:
        print("Error: Failure Messaging")
        return False

# Promotion Menu - finalize
def finalizePromotion(actionChoice):
    for item in promotionItems:
        print(colorize("[" + str(promotionItems.index(item)) + "] ", 'teal') + list(item)[0])
    while True:
        choice = input(">> ")
        try:
            choice = int(choice)
            if int(choice) < 0 or int(choice) > len(promotionItems): raise ValueError
            if str(promotionItems[int(choice)][1]) == "main":
                return action()
            choice = list(promotionItems[int(choice)])[1]
            return performPromotion(actionChoice, choice)
        except (ValueError, IndexError):
            settings.maybePrint(sys.exc_info()[0])
            print("Error: Incorrect Index")

def performPromotion(actionChoice, promotionChoice):
    def promote(username):
        if username == None:
            print("Warning: No user found")
        else:
            OnlySnarf.give_trial(username)
        mainMenu()    
    try:
        username = None
        if str(promotionChoice) == "email":
            # prompt
            choice = input("Email: ")
            username = str(choice)
            return promote(username)
        elif str(promotionChoice) == "select":
            users = displayUsers()    
            while True:
                choice = input(">> ")
                try:
                    if int(choice) < 0 or int(choice) > len(users): raise ValueError
                    if int(choice) == 0:
                        return finalizePromotion(actionChoice)
                    return promote(str(users[int(choice)-1].username))

                except (ValueError, IndexError):
                    settings.maybePrint(sys.exc_info()[0])
                    print("Error: Incorrect Index")            
    except (ValueError, IndexError):
        print("Error: Incorrect Index")
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Missing Method") 
    mainMenu()

# displays and returns users
def displayUsers():
    users = OnlySnarf.get_users()
    print(colorize("[0] ", 'blue') + "Back")
    # show list
    i = 1
    for user in users:
        print(colorize("[" + str(i) + "] ", 'blue') + str(user.username))
        i = i+1
    return users

def selectImage(folderName):
    images = Google.get_images()
    print(colorize("[0] ", 'blue') + "Back")
    i = 1
    for image in images:
        print(colorize("[" + str(i) + "] ", 'blue') + str(image[0]["title"]) + " - " + str(image[1]['title']))
        i = i+1
    return images
    # performMessage -> [folder , image_file] -> performMessage


###########################

### Settings Menu
def set_settings():
    showHeader()
    print(colorize("Set:",'menu'))
    global settingItems
    for item in settingItems:
        print(colorize("[" + str(settingItems.index(item)) + "] ", 'blue') + list(item)[0])
    while True:
        choice = input(">> ")
        try:
            if int(choice) < 0 or int(choice) >= len(settingItems): raise ValueError
            settingChoice = list(settingItems[int(choice)])[0]
            settingValue = list(settingItems[int(choice)])[1]
            if str(settingChoice) == "Back":
                return main()
            elif str(settingChoice) == "File Name":
                settingValue = input("Enter the file name: ")
            elif str(settingChoice) == "File Path":
                settingValue = input("Enter the file path: ")
            elif str(settingChoice) == "Text":
                settingValue = input("Enter the upload text: ")
            elif str(settingChoice) == "Mount Path":
                settingValue = input("Enter the mount path: ")
            elif str(settingChoice) == "Google: Root Folder Path":
                settingValue = input("Enter the drive path (folderName/folderName/...): ")
            elif str(settingChoice) == "Image":
                settingValue = input("Enter the image path: ")
            elif str(settingChoice) == "Google: Root Folder Name":
                settingValue = input("Enter the Google root folder name: ")
            elif str(settingChoice) == "Google: Drive Folders":
                settingValue = input("Enter the Google drive folders (separated by ',', no spaces): ")
                settingValue = settingValue.split(",")
            elif str(settingChoice) == "Image Limit":
                settingValue = input("Enter the image upload limit: ")
            elif str(settingChoice) == "Image Max":
                settingValue = input("Enter the image upload max: ")
            else:
                list_ = list(settingItems[int(choice)][2])
                print(colorize(str(settingChoice)+" =", 'blue'))
                for item in list_:
                    print(colorize("[" + str(list_.index(item)) + "] ", 'pink') + str(item))
                while True:
                    updateChoice = input(">> ")
                    try:
                        if int(updateChoice) < 0 or int(updateChoice) >= len(list(settingItems[int(choice)][2])): raise ValueError
                        settingValue = list_[int(updateChoice)]
                        break
                    except (ValueError, IndexError):
                        print("Error: Incorrect Index")
                    except Exception as e:
                        settings.maybePrint(e)
                        break
            global UPDATED
            UPDATED = settingChoice
            global UPDATED_TO
            UPDATED_TO = settingValue
            settingItems[int(choice)][1] = settingValue
            settings.update_value(settingChoice, settingValue)
            return set_settings()
        except (ValueError, IndexError):
            print("Error: Incorrect Index")
        except Exception as e:
            settings.maybePrint(e)
            return main()

###########################

def colorize(string, color):
    if not color in colors: return string
    return colors[color] + string + '\033[0m'
  
def exit():
    print("Shnarrf?")
    sys.exit(0)

###########################
import atexit
def exit_handler():
    print('Shnnarrrff!')
    exit()
atexit.register(exit_handler)

import signal
def signal_handler(sig, frame):
    print('Shnnnarf?')
    exit()
signal.signal(signal.SIGINT, signal_handler)
###########################

def main():
    showHeader()
    mainMenu()

def mainMenu():
    ### Main Menu
    print(colorize("Select an option:", 'menu'))
    for item in menuItems:
        print(colorize("[" + str(menuItems.index(item)) + "] ", 'blue') + list(item)[0])
    while True:
        choice = input(">> ")
        try:
            if int(choice) < 0 or int(choice) >= len(menuItems): raise ValueError
            # Call the matching function
            method_name = list(menuItems[int(choice)])[1]
            possibles = globals().copy()
            possibles.update(locals())
            method = possibles.get(method_name)
            if method is not None:
                return method()
            else:
                print("Error: Missing Option")    
        except (ValueError, IndexError, KeyboardInterrupt):
            print("Error: Incorrect Index")
            pass

def showHeader():
    os.system('clear')
    # Print some badass ascii art header here !
    print(colorize(header, 'header'))
    print(colorize('version '+version+'\n', 'green'))
    showSettings()

def showSettings():
    print('Settings:')
    for setting in settingItems:
        if str(setting[0]) == "Image Limit" and setting[3]:
            print(' - '+setting[0]+' = '+str(setting[1])+"/"+str(settings.IMAGE_UPLOAD_MAX))
        elif str(setting[0]) != "Back" and str(settings.DEBUG) == "True":
            print(' - '+setting[0]+' = '+str(setting[1]))
        elif str(setting[0]) != "Back" and setting[3]:
            print(' - '+setting[0]+' = '+str(setting[1]))
        
    global UPDATED
    global UPDATED_TO
    if str(UPDATED) != "False":
        print('\nUpdated: '+str(UPDATED)+' -> '+str(UPDATED_TO))
    UPDATED = False
    print('\r')

###########################

if __name__ == "__main__":
    try:
        main_other()
    except:
        # print(sys.exc_info()[0])
        print("Shhhhhnnnnnarf!")
    finally:
        sys.exit(0)

def main_other():
    settings.initialize()
    initialize()
    main()


from crontab import CronTab
from OnlySnarf.settings import SETTINGS as settings
    
def deleteCron(comment):
    cron = findCron(comment)
    cron.remove(cron)  

def deleteAllCrons():
    cron = CronTab(user=str(settings.USER))
    cron.remove_all()  

# use cron.comment to set cron name and find crons
def disableCron(comment):
    # find cron by comment
    cron = findCron(comment)
    cron.enable(False)

def enableCron(comment):
    # find cron by comment
    cron = findCron(comment)
    cron.enable()

def createCron(comment,  minute=None, hour=None):
    print("Creating Cron: {}".format(comment))
    if findCron(comment) is not None:
        print("Warning: Cron Exists")
        return
    cron = CronTab(user=str(settings.USER))
    newCron = cron.new(command='onlysnarf -cron -{}'.format(comment), comment=comment);
    newCron.hour.every(1)
    if minute is not None:
        newCron.minute.on(minute)
    if hour is not None:
        newCron.hour.on(hour)
    cron.write()
    for item in cron:  
        print(item)
    print("Created Cron: {}".format(comment))

def listCrons():
    cron = CronTab(user=str(settings.USER))
    for job in cron:
        print(job)

# cron.find_command("command name")
# cron.find_comment("comment")
# cron.find_time(time schedule)
def findCron(comment):
    cron = CronTab(user=str(settings.USER))
    print(cron.find_comment(str(comment)))
    if str(settings.DEBUG) == "True":
        return None
    return True 

###################
##### Special #####
###################

# sends messages to users that were queued for a specific time
def sendQueuedMessages():
    pass


def test():
    createCron("upload-video")
    createCron("upload-video", "30", "11")
    createCron("check-scenes")


#!/usr/bin/python3
# 3/28/2019 Skeetzo

import random
import os
import shutil
import datetime
import json
import sys
import pathlib
import time
from OnlySnarf.settings import SETTINGS as settings
from OnlySnarf import google as Google
from OnlySnarf import driver as OnlySnarf
from OnlySnarf import cron as Cron
# from pprint import pprint

##########################
##### MENU FUNCTIONS #####
##########################

def all(opt):
    main(opt)

####################
##### Download #####
####################

def download(fileChoice):
    if fileChoice == 'image':
        return download_random_image()
    elif fileChoice == 'gallery':
        return download_random_gallery()
    elif fileChoice == 'performer':
        return download_random_performer()
    elif fileChoice == 'scene':
        return download_random_scene()
    elif fileChoice == 'video':
        return download_random_video()
    else:
        return print("Error: Missing File Choice")

def download_message_image(folderName="random"):
    remove_local()
    print('Fetching Image')
    response = Google.get_message_image(folderName)
    if response == None:
        print("Error: Missing Image Download")
        return
    google_file = response[0]
    folder_name = response[1]
    if google_file == None:
        return
    file_name = google_file['title']
    file_path = Google.download_file(google_file)
    if google_file == None:
        print('Error: Missing Random Image')
        return
    if file_path == None:
        print('Error: Empty Download')
        return
    return [file_name, file_path, google_file, folder_name]

def download_random_image():
    remove_local()
    print('Fetching Image')
    response = Google.get_random_image()
    if response == None:
        print("Error: Missing Image Download")
        return
    google_file = response[0]
    folder_name = response[1]
    if google_file == None:
        return
    file_name = google_file['title']
    file_path = Google.download_file(google_file)
    if google_file == None:
        print('Error: Missing Random Image')
        return
    if file_path == None:
        print('Error: Empty Download')
        return
    return [file_name, file_path, google_file, folder_name]

def download_random_gallery():
    remove_local()
    print('Fetching Gallery')
    response = Google.get_random_gallery()
    if response == None:
        print("Error: Missing Gallery Download")
        return
    google_file = response[0]
    folder_name = response[1]
    if google_file == None:
        print("Error: Missing Google File")
        return
    file_name = google_file['title']
    results = Google.download_gallery(google_file)
    gallery_files = results[0]
    file_path = results[1]
    if file_path == None:
        print('Error: Missing Random Gallery')
        return
    return [file_name, file_path, google_file, gallery_files, folder_name]

def download_random_performer():
    remove_local()
    print('Fetching Performer')
    google_file = Google.get_random_performer()
    if google_file == None:
        print("Error: Missing Performer")
        return
    performer = google_file['title']
    results = Google.download_performer(google_file)
    if results == None:
        print("Error: Missing Performer Folders")
        return
    gallery_files = results[0]
    file_path = results[1]
    gallery_name = results[2]
    if file_path == None:
        print('Error: Missing Content')
        return
    if gallery_files == None:
        print('Error: Missing Gallery Content')
        return
    if performer == None:
        print('Error: Missing Performer Name')
        return
    if gallery_name == None:
        print('Error: Missing Gallery Name')
        return
    return [file_path, google_file, performer, gallery_name, google_file]

def download_random_video():
    remove_local()
    print('Fetching Video')
    response = Google.get_random_video()
    if response == None:
        print("Error: Missing Video Download")
        return
    google_file = response[0]
    folder_name = response[1]
    if google_file == None:
        return
    file_name = google_file['title']
    repair = False
    if str(folder_name) == "gopro":
        repair = True
    file_path = Google.download_file(google_file, REPAIR=repair)
    if google_file == None:
        print('Error: Missing Random Video')
        return
    if file_path == None:
        print('Error: Empty Download')
        return
    return [file_name, file_path, google_file, folder_name]

def download_random_scene():
    remove_local()
    print('Fetching Scene')
    response = Google.get_random_scene()
    if response == None:
        print("Error: Missing Scene Download")
        return
    google_file = response[0]
    folder_name = response[1]
    if google_file == None:
        print("Error: Missing Google File")
        return
    file_name = google_file['title']
    results = Google.download_scene(google_file)
    if results == None:
        print('Error: Empty Download')
        return
    return results

###################
##### Message #####
###################

def message(choice, message=None, image=None, price=None, username=None):
    if not image[0] or image[0] == None:
        print("Error: Missing Image")
        return False
    successful_message = OnlySnarf.message(choice=choice, message=message, image=image, price=price, username=username)
    return successful_message

#####################
##### Promotion #####
#####################

def give_trial(user):
    print("Applying Promotion: "+user)
    link = OnlySnarf.get_new_trial_link()
    text = "Here's your free trial link!\n"+link
    settings.maybePrint("Link: "+str(text))
    send_email(email, text)

def send_email(email, text):
    print("Sending Email: "+str(email))
    pass

#################
##### Reset #####
#################

# Deletes local file
def remove_local():
    try:
        if str(settings.SKIP_DELETE) == "False":
            print("Skipping Local Remove")
            return
        # print('Deleting Local File(s)')
        # delete /tmp
        tmp = settings.getTmp()
        if os.path.exists(tmp):
            shutil.rmtree(tmp)
            print('Local File(s) Removed')
        else:
            print('Local Files Not Found')
    except OSError as e:
        print("Error: Missing Local Path")
    except Exception as e:
        print(e)

###################
##### Release #####
###################

def release(opt):
    print("0/3 : Deleting Locals")
    remove_local()
    sys.stdout.flush()
    #################################################
    print("1/3 : Running - {}".format(opt))
    if str(opt) == "image":
        released = release_image()
    elif str(opt) == "video":
        released = release_video()
    elif str(opt) == "gallery":
        released = release_gallery()
    elif str(opt) == "performer":
        released = release_performer()
    elif str(opt) == "scene":
        released = release_scene()
    else:
        print('Missing Args!')
        return
    sys.stdout.flush()
    if released == False:
        print("Error: Failed to release - {}".format(opt))
        return
    #################################################
    print('2/3 : Cleaning Up Files')
    remove_local()
    print('Files Cleaned ')
    #################################################
    print('3/3 : Google Drive to OnlyFans Upload Complete')
    sys.stdout.flush()
    OnlySnarf.exit()

def release_image():
    try:
        print("Releasing Image")
        response = download("image")
        if response == None:
            print("Error: Failure Releasing Image")
            return False
        # settings.maybePrint("Image: {}".format(response))
        text = None
        content = None
        file = None
        keywords = None
        try:
            text = response[0]
            content = response[1]
            file = response[2]
            keywords = response[3].split(" ")
        except Exception as e:
            settings.maybePrint(e)
        if str(keywords) == " " or str(keywords[0]) == " ":
            keywords = []
        if text == None:
            print("Error: Missing Image Title")
            return False
        if content == None:
            print("Error: Missing Image Content")
            return False
        if file == None:
            print("Error: Missing Image File")
            return False
        if keywords == None:
            print("Warning: Missing Image Keywords")
        print("Image:")
        print("- Title: {}".format(text)) # name of scene
        print("- Keywords: {}".format(keywords)) # text sent in messages
        print("- Content: {}".format(content)) # the file(s) to upload
        successful = upload("image", path=content, text=text, keywords=keywords)
        if successful:
            Google.move_file(file)
        else:
            return False
        return True
    except Exception as e:
        settings.maybePrint(e)
        return False

def release_gallery():
    try:
        print("Releasing Gallery")
        response = download("gallery")
        if response == None:
            print("Error: Failure Releasing Gallery")
            return False
        # settings.maybePrint("Gallery: {}".format(response))
        text = None
        content = None
        google_file = None
        google_files = None
        keywords = None
        try:
            text = response[0]
            content = response[1]
            google_file = response[2]
            google_files = response[3]
            keywords = response[4].split(" ")
        except Exception as e:
            settings.maybePrint(e)
        if keywords == " " or str(keywords[0]) == " ":
            keywords = []
        if text == None:
            print("Error: Missing Gallery Title")
            return False
        if content == None:
            print("Error: Missing Gallery Content")
            return False
        if google_file == None:
            print("Error: Missing Gallery File")
            return False
        if google_files == None or len(google_files) == 0:
            print("Error: Missing Gallery Files")
            return False
        if str(keywords) == None:
            print("Warning: Missing Gallery Keywords")
        print("Gallery:")
        print("- Title: {}".format(text)) # name of scene
        print("- Keywords: {}".format(keywords)) # text sent in messages
        print("- Content: {}".format(content)) # the file(s) to upload
        files = os.listdir(content)
        file = files[0]
        ext = str(os.path.splitext(file)[1].lower())
        settings.maybePrint('ext: '+str(ext))
        successful = upload("gallery", path=content, text=text, keywords=keywords)
        if successful:
            Google.move_files(google_file['title'], google_files)
        else:
            return False
        return True
    except Exception as e:
        settings.maybePrint(e)
        return False

def release_performer():
    try:
        print("Releasing Performer")
        response = download("performer")
        if response == None:
            print("Error: Failure Releasing Performer")
            return False
        # settings.maybePrint("Performer: {}".format(response))
        text = None
        performer = None
        content = None
        google_file = None
        folder_name = None
        try:
            content = response[0]
            google_file = response[1]
            performer = response[2]
            text = response[3]
            folder_name = response[4]
        except Exception as e:
            settings.maybePrint(e)
        if text == None:
            print("Error: Missing Performer Text")
            return False
        if performer == None:
            print("Error: Missing Performer Name")
            return False
        if content == None:
            print("Error: Missing Performer Content")
            return False
        text += " w/ @{}".format(performer)
        print("Performer:")
        print("- Performer: {}".format(performer)) # name of scene
        print("- Text: {}".format(text)) # name of scene
        print("- Content: {}".format(content)) # the file(s) to upload
        files = os.listdir(content)
        file = files[0]
        ext = str(os.path.splitext(file)[1].lower())
        settings.maybePrint('ext: '+str(ext))
        if ext == ".mp4":
            successful = upload("video", path=content, text=text) 
        else:
            successful = upload("gallery", path=content, text=text)
        if successful:
            Google.move_file(google_file)
        else:
            return False
        return True
    except Exception as e:
        settings.maybePrint(e)
        return False
    
# upload a file or gallery
# send a message to [recent, all, user] w/ a preview image
def release_scene():
    try:
        print("Releasing Scene")
        response = download("scene")
        if response == None:
            print("Error: Failure Releasing Scene")
            return False
        # settings.maybePrint("Scene: {}".format(response))
        content = response[0]
        preview = response[1]
        data = response[2]
        google_folder = response[3]
        # print("Data:\n{}".format(json.dumps(data, sort_keys=True, indent=4)))
        data = json.loads(json.dumps(data))
        settings.maybePrint("Data: {}".format(data))
        title = None
        message = None
        price = None
        text = None
        performers = None
        keywords = None
        users = None
        title = data["title"]
        message = data["message"]
        price = data["price"]
        text = data["text"]
        performers = data["performers"]
        keywords = data["keywords"]
        if str(keywords) == " " or str(keywords[0]) == " ":
            keywords = []
        users = data["users"]
        if title == None:
            print("Error: Missing Scene Title")
            return False
        if message == None:
            print("Error: Missing Scene Message")
            return False
        if price == None:
            print("Error: Missing Scene Price")
            return False
        if text == None:
            print("Error: Missing Scene Text")
            return False
        print("Scene:")
        print("- Title: {}".format(title)) # name of scene
        print("- Text: {}".format(text)) # text entered into file upload
        print("- Price: {}".format(price)) # price of messages sent
        print("- Message: {}".format(message)) # text sent in messages
        print("- Keywords: {}".format(keywords)) # text sent in messages
        print("- Performers: {}".format(performers)) # text sent in messages
        print("- Preview: {}".format(preview)) # image sent in messages
        print("- Content: {}".format(content)) # the file(s) to upload
        print("- Users: {}".format(users)) # the file(s) to upload 
        files = os.listdir(content)
        file = files[0]
        ext = str(os.path.splitext(file)[1].lower())
        settings.maybePrint('ext: '+str(ext))
        successful_upload = False
        if not ext or ext == '.mp4':
            successful_upload = upload("video", path=content, text=text, keywords=keywords, performers=performers)
        elif ext == '.jpg' or ext == '.jpeg' and len(files) > 1:
            successful_upload = upload("gallery", path=content, text=text, keywords=keywords, performers=performers)
        elif ext == '.jpg' or ext == '.jpeg' and len(files) == 1:
            successful_upload = upload("image", path=content, text=text, keywords=keywords, performers=performers) 
        else:
            print("Error: Missing Scene Type")
        if successful_upload:
            if str(users[0]) == "all" or str(users[0]) == str("recent") or str(users[0]) == str("favorites"):
                users = users[0]
            if not users or str(users).lower() == "none":
                print("Warning: Missing User Choice")
            elif str(users) == "all" or str(users) == "recent" or str(users) == "favorites":
                successful_message = OnlySnarf.message(choice=str(users), message=message, image=preview, price=price)
            else:
                for user in users:
                    successful_message = OnlySnarf.message(choice="user", message=message, image=preview, price=price, username=user)
            if successful_message:
                Google.move_file(google_folder)
            else:
                print("Error: Failure Messaging")
                return False
        else:
            print("Error: Failure Uploading")
            return False
        return True
    except Exception as e:
        settings.maybePrint(e)
        return False

def release_video():
    try:
        print("Releasing Video")
        response = download("video")
        if response == None:
            print("Error: Failure Releasing Video")
            return False
        # settings.maybePrint("Video: {}".format(response))
        text = None
        content = None
        file = None
        keywords = None
        try:
            text = response[0]
            content = response[1]
            file = response[2]
            keywords = response[3].split(" ")
        except Exception as e:
            settings.maybePrint(e)
        if str(keywords) == " " or str(keywords[0]) == " ":
            keywords = []
        ext = str(os.path.splitext(text)[1].lower())
        settings.maybePrint('ext: '+str(ext))
        if ext == '.mp4':
            settings.maybePrint("pruning extension")
            text = os.path.splitext(text)[0]
        if text == None:
            print("Error: Missing Video Title")
            return False
        if content == None:
            print("Error: Missing Video Content")
            return False
        if file == None:
            print("Error: Missing Video File")
            return False
        if keywords == None:
            print("Warning: Missing Video Keywords")
        print("Video:")
        print("- Title: {}".format(text)) # name of scene
        print("- Keywords: {}".format(keywords)) # text sent in messages
        print("- Content: {}".format(content)) # the file(s) to upload
        successful = upload("video", path=content, text=text, keywords=keywords)
        if successful:
            Google.move_file(file)
        else:
            return False
        return True
    except Exception as e:
        settings.maybePrint(e)
        return False

##################
##### Upload #####
##################

def upload(fileChoice, path=None, text=None, keywords=None, performers=None):
    settings.maybePrint("Uploading: {} - {} - {}".format(fileChoice, path, text))
    successful = False
    try:
        if fileChoice == 'image':
            successful = OnlySnarf.upload_to_OnlyFans(path=path, text=text, keywords=keywords, performers=performers)
        elif fileChoice == 'gallery':
            successful = OnlySnarf.upload_to_OnlyFans(path=path, text=text, keywords=keywords, performers=performers)
        elif fileChoice == 'video':
            successful = OnlySnarf.upload_to_OnlyFans(path=path, text=text, keywords=keywords, performers=performers)
        else:
            print("Missing Upload Choice")
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Unable to Upload")
        return
    if successful:
        print("Upload Complete")
    else:
        print("Upload Failure")
    return successful

#################
##### Users #####
#################

def get_users():
    settings.maybePrint("Getting Users")
    successful = False
    try:
        successful = OnlySnarf.get_users()
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Unable to get users");
        return
    if successful:
        settings.maybePrint("Users Found")
    else:
        print("Warning: users not found")
        return []
    return successful

#####################
##### FUNCTIONS #####
#####################

def main(opt):
    print("0/3 : Deleting Locals")
    remove_local()
    sys.stdout.flush()
    #################################################
    print("1/3 : Running - {}".format(opt))
    if str(opt) == "image":
        released = release_image()
    elif str(opt) == "video":
        released = release_video()
    elif str(opt) == "gallery":
        released = release_gallery()
    elif str(opt) == "performer":
        released = release_performer()
    elif str(opt) == "scene":
        released = release_scene()
    else:
        print('Missing Args!')
        return
    sys.stdout.flush()
    if released == False:
        print("Error: Failed to release - {}".format(opt))
        return
    #################################################
    print('2/3 : Cleaning Up Files')
    remove_local()
    print('Files Cleaned ')
    #################################################
    print('3/3 : Google Drive to OnlyFans Upload Complete')
    sys.stdout.flush()
    OnlySnarf.exit()

def test(TYPE):
    print('0/3 : Deleting Locals')
    remove_local()
    print('1/3 : Testing')



    # ### Promotion ###
    print('TESTING: Cron')
    response = Cron.test()
    if not response or response == None:
        print("Error: Failed to test crons")
    reset = OnlySnarf.reset()
    if not reset:
        return print("Error: Failed to Reset")
    return


    # ### Promotion ###
    print('TESTING: Promotion')
    response = apply_promotion()
    if not response or response == None:
        print("Error: Failed to apply promotion")
    reset = OnlySnarf.reset()
    if not reset:
        return print("Error: Failed to Reset")
    return

    # ### Gallery ###
    # print('TESTING: Gallery x 10')
    # for i in range(10):
    #     release_gallery()
    #     time.sleep(10)
    #     reset = OnlySnarf.reset()
    #     if not reset:
    #         return print("Error: Failed to Reset")
    # return sys.exit(0)
    # #######################

    # ### Message ###
    # response = download_random_image()
    # if not response or response == None:
        # print("Error: Missing Image")
        # return
    # successful_message = OnlySnarf.message(choice="all", message="random tease :P", image=response[1], price="5.00")
    # message(choice="recent", message="8=======D", image=response[1], price="50.00")
    # if successful_message:
        # Google.move_file(response[2])
    # else:
        # print("Error: Failed to Send Message")
    # #######################
    # ### Exit Gracefully ###
    # OnlySnarf.exit()
    # return
    # #######################

    # ### Users ###
    print('TESTING: Users')
    users = OnlySnarf.get_users()
    time.sleep(30)
    reset = OnlySnarf.reset()
    if not reset:
        return print("Error: Failed to Reset")
    return
    #######################

    # ### Chat Logs ###
    # print('TESTING: Chat Logs')
    # OnlySnarf.update_chat_logs()
    # time.sleep(30)
    # reset = OnlySnarf.reset()
    # if not reset:
    #     return print("Error: Failed to Reset")
    # #######################
    # ### Exit Gracefully ###
    # OnlySnarf.exit()
    # return
    # #######################
    
    # ### Image ###
    # print('TESTING: Image')
    # release_image()
    # time.sleep(30)
    # reset = OnlySnarf.reset()
    # if not reset:
    #     return print("Error: Failed to Reset")
    # ### Gallery ###
    # print('TESTING: Gallery')
    # release_gallery()
    # time.sleep(30)
    # reset = OnlySnarf.reset()
    # if not reset:
    #     return print("Error: Failed to Reset")
    # ### Performer ###
    # print('TESTING: Performer')
    # release_performer()
    # time.sleep(30)
    # reset = OnlySnarf.reset()
    # if not reset:
    #     return print("Error: Failed to Reset")
    ### Scene ###
    # print('TESTING: Scene')
    # release_scene()
    # time.sleep(30)
    # reset = OnlySnarf.reset()
    # if not reset:
    #     return print("Error: Failed to Reset")
    # ### Video ###
    # print('TESTING: Video')
    # release_video()
    # time.sleep(30)
    # reset = OnlySnarf.reset()
    # if not reset:
    #     return print("Error: Failed to Reset")
    
    #######################
    ### Exit Gracefully ###
    # OnlySnarf.exit()
    #######################


################################################################################################################################################

if __name__ == "__main__":
    try:
        # os.system('clear')
        settings.initialize()
        OnlySnarf.initialize()
        main(settings.TYPE)
    except:
        print(sys.exc_info()[0])
        print("Shnarf!")
    finally:
        sys.exit(0)
else:
    try:
        settings.initialize()
        OnlySnarf.initialize()
    except Exception as e:
        print(e)
        print("Shnnarf?")


#!/usr/bin/python3
# 3/28/2019: Skeetzo

import random
import os
import shutil
import datetime
import json
import sys
import subprocess
import pathlib
import io
from subprocess import PIPE, Popen
from pydrive.auth import GoogleAuth
from pydrive.drive import GoogleDrive
from moviepy.editor import VideoFileClip
from apiclient.discovery import build
from httplib2 import Http
from oauth2client import file, client, tools
from apiclient.http import MediaFileUpload,MediaIoBaseDownload
from .settings import SETTINGS as settings

###################
##### Globals #####
###################

AUTH = False
DRIVE = None
PYDRIVE = None
ONE_GIGABYTE = 1000000000
ONE_MEGABYTE = 1000000
FIFTY_MEGABYTES = 50000000
ONE_HUNDRED_KILOBYTES = 100000
FOLDERS = None
OnlyFansFolder_ = None

################
##### Auth #####
################

# Google Auth
def authGoogle():
    print('Authenticating Google')
    try:
        # PyDrive
        gauth = GoogleAuth()
        # Try to load saved client credentials
        gauth.LoadCredentialsFile(settings.PATH_GOOGLE_CREDS)
        settings.maybePrint('Loaded: Google Credentials')
        if gauth.credentials is None:
            # Authenticate if they're not there
            gauth.LocalWebserverAuth()
        elif gauth.access_token_expired:
            # Refresh them if expired
            gauth.Refresh()
        else:
            # Initialize the saved creds
            gauth.Authorize()
        # Save the current credentials to a file
        gauth.SaveCredentialsFile(settings.PATH_GOOGLE_CREDS)
        global PYDRIVE
        PYDRIVE = GoogleDrive(gauth)
        # Drive v3 API
        SCOPES = 'https://www.googleapis.com/auth/drive'
        store = file.Storage(settings.PATH_GOOGLE_CREDS)
        creds = store.get()
        if not creds or creds.invalid:
            flow = client.flow_from_clientsecrets(settings.PATH_SECRET, SCOPES)
            creds = tools.run_flow(flow, store)
        global DRIVE
        DRIVE = build('drive', 'v3', http=creds.authorize(Http()))
    except:
        settings.maybePrint(sys.exc_info()[0])
        print('Error: Unable to Authenticate w/ Google')
        return False
    print('Authentication Success') 
    return True

################################
##### Archiving / Deleting #####
################################

# Deletes online file
def delete_file(file):
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    if str(settings.FORCE_DELETE_GOOGLE) == "True":
        print("Deleting (Forced): {}".format(fileName))
    elif str(settings.DEBUG) == "True":
        print("Skipping Delete (Debug): {}".format(fileName))
        return
    elif str(settings.DELETE_GOOGLE) == "False":
        print('Skipping Delete (Disabled): {}'.format(fileName))
        return
    else:
        print('Deleting: {}'.format(fileName))
    file.Trash()

# Archives posted file / folder
def move_file(file):
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    if str(settings.FORCE_BACKUP) == "True":
        print("Backing Up (Forced): {}".format(file['title']))
    elif str(settings.DEBUG) == "True":
        print("Skipping Backup (Debug): {}".format(file['title']))
        return
    elif str(settings.BACKUP) == "False":
        print('Skipping Backup (Disabled): {}'.format(file['title']))
        return
    else:
        print('Backing Up: {}'.format(file['title']))
    OnlyFans_POSTED_FOLDER = get_folder_by_name("posted")
    file['parents'] = [{"kind": "drive#fileLink", "id": str(OnlyFans_POSTED_FOLDER)}]
    file.Upload()
    print('Google File Backed Up: {}'.format(file['title']))

def move_files(fileName, files):
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    if str(settings.FORCE_BACKUP) == "True":
        print("Backing Up (Forced): {}".format(fileName))
    elif str(settings.DEBUG) == "True":
        print("Skipping Backup (Debug): {}".format(fileName))
        return
    elif str(settings.BACKUP) == "False":
        print('Skipping Backup (Disabled): {}'.format(fileName))
        return
    else:
        print('Backing Up: {}'.format(fileName))
    title = fileName+" - "+datetime.datetime.now().strftime("%d-%m@%I-%M")
    settings.maybePrint('Moving To: '+title)
    global PYDRIVE
    OnlyFans_POSTED_FOLDER = get_folder_by_name("posted")
    tmp_folder = PYDRIVE.CreateFile({'title':str(title), 'parents':[{"kind": "drive#fileLink", "id": str(OnlyFans_POSTED_FOLDER)}],'mimeType':'application/vnd.google-apps.folder'})
    tmp_folder.Upload()
    settings.maybePrint("Backing Up:")
    for file in files:
        settings.maybePrint(" - {}".format(file['title']))
        file['parents'] = [{"kind": "drive#fileLink", "id": tmp_folder['id']}]
        file.Upload()
    print('Google Files Backed Up: {}'.format(title))

###################
##### Folders #####
###################

def create_folders():
    print("Creating Folders: {}".format(settings.ROOT_FOLDER))
    OnlyFansFolder = get_folder_root()
    if OnlyFansFolder is None:
        print("Error: Unable To Create Folders")
        return
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(OnlyFansFolder)}).GetList()
    for folder in settings.DRIVE_FOLDERS:
        found = False
        for folder_ in file_list:
            if str(folder) == folder_['title']:
                settings.maybePrint("found: {}".format(folder))
                found = True
        if not found:
            settings.maybePrint("created: {}".format(folder))
            contentFolder = PYDRIVE.CreateFile({"title": str(folder), "parents": [{"id": OnlyFansFolder}], "mimeType": "application/vnd.google-apps.folder"})
            contentFolder.Upload()

def find_folder(parent, folderName):
    global PYDRIVE
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(parent)}).GetList()
    for folder in file_list:
        if str(folder['title']) == str(folderName):
            return folder['id']
    print("Error: Unable to Find Folder - {}".format(folderName))
    return None

def get_folder_by_name(folderName):
    settings.maybePrint("Getting Folder: {}".format(folderName))
    global PYDRIVE
    global FOLDERS
    if FOLDERS is not None:
        if FOLDERS.get(str(folderName)) is not None:
            return FOLDERS.get(str(folderName))
    else:
        FOLDERS = {}
    OnlyFansFolder = get_folder_root()
    if OnlyFansFolder is None:
        print("Error: Unable To Get Folder - {}".format(folderName))
        return None
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(OnlyFansFolder)}).GetList()
    for folder in file_list:
        if str(folder['title'])==str(folderName):
            FOLDERS[str(folderName)] = folder['id']
            settings.maybePrint("Found Folder: {}".format(folderName))
            return folder['id']
    if str(settings.DRIVE_CREATE_MISSING) == "False":
        settings.maybePrint("Skipping: Create Missing Folder - {}".format(folderName))
        return None
    # create if missing
    folder = PYDRIVE.CreateFile({"title": str(folderName), "mimeType": "application/vnd.google-apps.folder", "parents": [{"kind": "drive#fileLink", "id": OnlyFansFolder}]})
    folder.Upload()
    FOLDERS[str(folderName)] = folder['id']
    settings.maybePrint("Created Folder: {}".format(folderName))
    return folder['id']

# Creates the OnlyFans folder structure
def get_folder_root():
    global PYDRIVE
    global OnlyFansFolder_
    if OnlyFansFolder_ is not None:
        return OnlyFansFolder_
    OnlyFansFolder = None
    if settings.PATH_DRIVE is not None:
        mount_root = "root"
        root_folders = settings.PATH_DRIVE.split("/")
        settings.maybePrint("Mount Folders: {}".format(root_folders))    
        for folder in root_folders:
            mount_root = find_folder(mount_root, folder)
            if mount_root is None:
                mount_root = "root"
                print("Warning: Drive Mount Folder Not Found")
                break
        mount_root = find_folder(mount_root, settings.ROOT_FOLDER)
        if mount_root is None:
            mount_root = "root"
            print("Warning: Drive Mount Folder Not Found")
        else:
            print("Found Root (alt): {}/{}".format(settings.PATH_DRIVE, settings.ROOT_FOLDER))
        OnlyFansFolder = mount_root
    else:
        file_list = PYDRIVE.ListFile({'q': "'root' in parents and trashed=false"}).GetList()
        for folder in file_list:
            if str(folder['title']) == str(settings.ROOT_FOLDER):
                OnlyFansFolder = folder['id']
                print("Found Root: {}".format(settings.ROOT_FOLDER))
    if OnlyFansFolder is None:
        print("Creating Root: {}".format(settings.ROOT_FOLDER))
        OnlyFansFolder = PYDRIVE.CreateFile({"title": str(settings.ROOT_FOLDER), "mimeType": "application/vnd.google-apps.folder"})
        OnlyFansFolder.Upload()
        OnlyFansFolder = OnlyFansFolder['id']
    OnlyFansFolder_ = OnlyFansFolder
    return OnlyFansFolder_

####################
##### Download #####
####################

# Download File
def download_file(file, REPAIR=False):
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Downloading File')
    tmp = settings.getTmp()
    # download file
    name = os.path.splitext(file['title'])[0]
    ext = os.path.splitext(file['title'])[1].lower()
    if not ext:
        ext = '.mp4'
        settings.maybePrint('ext (default): '+str(ext))
    else:
        settings.maybePrint('ext: '+str(ext))
    name = "{}{}".format(name, ext)
    tmp = os.path.join(tmp, name)
    settings.maybePrint("name: {}".format(name))
    settings.maybePrint('path: '+str(tmp))
    if str(ext).lower() == ".mp4":
        with open(tmp, 'w+b') as output:
            # print("8",end="",flush=True)
            file_id = file['id']
            request = DRIVE.files().get_media(fileId=file['id'])
            downloader = MediaIoBaseDownload(output, request)
            # print("=",end="",flush=True)
            done = False
            while done is False:
                # print("=",end="",flush=True)
                status, done = downloader.next_chunk()
                if str(settings.VERBOSE) == "True":
                    print("Downloading: %d%%\r" % (status.progress() * 100),end="")
            # print("D")
            print("Download Complete")
        if REPAIR:
            tmp = repair(tmp)
        global FIFTY_MEGABYTES
        if int(os.stat(str(tmp)).st_size) >= FIFTY_MEGABYTES or settings.FORCE_REDUCTION: # greater than 1GB
            tmp = reduce(tmp)
        tmp = thumbnail_fix(tmp)
    else:
        file.GetContentFile(tmp)
    ### Finish ###
    if not os.path.isfile(str(tmp)):
        print("Error: Missing Downloaded File")
        return
    size = os.path.getsize(tmp)
    settings.maybePrint("File Size: {}kb - {}mb".format(size/1000, size/1000000))
    global ONE_MEGABYTE
    if size <= ONE_MEGABYTE:
        settings.maybePrint("Warning: Small File Size")
    global ONE_HUNDRED_KILOBYTES
    if size <= ONE_HUNDRED_KILOBYTES:
        settings.maybePrint("Error: File Size Too Small")
        print("Error: Download Failure")
        return
    print('Downloaded: File')
    return tmp

# Download Gallery
def download_gallery(folder):
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Downloading Gallery')
    tmp = settings.getTmp()
    # download folder
    global PYDRIVE
    file_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\')"}).GetList()
    folder_size = len(file_list)
    settings.maybePrint('Folder size: '+str(folder_size))
    settings.maybePrint('Upload limit: '+str(settings.IMAGE_UPLOAD_LIMIT))
    if int(folder_size) == 0:
        print('Error: Empty Folder')
        return
    random.shuffle(file_list)
    file_list = file_list[:int(settings.IMAGE_UPLOAD_LIMIT)]
    i = 1
    for file in sorted(file_list, key = lambda x: x['title']):
        print('Downloading: {} ({}/{})'.format(file['title'], i, folder_size))
        settings.maybePrint('filePath: '+os.path.join(tmp, str(file['title'])))
        file.GetContentFile(os.path.join(tmp, str(file['title'])))
        i+=1
    print('Downloaded: Gallery')
    return [file_list, tmp]

# Download Performer
def download_performer(folder):
    print('Downloading Performer: {}'.format(folder['title']))
    # mkdir /tmp
    tmp = settings.getTmp()
    global PYDRIVE
    content_folders = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    content_found = []
    random_content = None
    content_title = None
    for folder in content_folders:
        settings.maybePrint('content: '+folder['title'])
        content_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
        if len(content_list)==0:
            settings.maybePrint('- skipping empty content: '+folder['title'])
        elif len(content_list)>0:
            settings.maybePrint('- content galleries found: '+folder['title'])
            for content in content_list:
                content_list_galleries = PYDRIVE.ListFile({'q': "'"+content['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\' or mimeType contains \'video/mp4\')"}).GetList()
                if len(content_list_galleries)==0:
                    settings.maybePrint('- skipping empty content gallery: '+content['title'])
                elif len(content_list_galleries)>0 and len(content_list_galleries):
                    settings.maybePrint('- content gallery found: '+content['title'])
                    content_found.append(content)
    if len(content_found)==0:
        print('Warning: Missing Content Folder')
        return
    random_content = random.choice(content_found)
    content_title = random_content['title']
    settings.maybePrint("Folder: {}".format(content_title))
    # download folder
    file_list = PYDRIVE.ListFile({'q': "'"+random_content['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\' or mimeType contains \'video/mp4\')"}).GetList()
    folder_size = len(file_list)
    settings.maybePrint('Folder size: {}'.format(folder_size))
    settings.maybePrint('Upload limit: {}'.format(settings.IMAGE_UPLOAD_LIMIT))
    # settings.maybePrint("Files: {}".format(file_list))
    if folder_size == 0:
        print("Error: Missing Files")
        return
    # get all images first then videos 1 at a time from folder
    videos = []
    images = []
    for file in file_list:
        if "mp4" in str(file['title']):
            videos.append(file)
        else:
            images.append(file)
    if len(images) > len(videos):
        settings.maybePrint("Found: Images")
        file_list_random = []
        for x in range(folder_size):
            random_file = random.choice(file_list)
            file_list.remove(random_file)
            file_list_random.append(random_file)
        file_list = file_list_random[:settings.IMAGE_UPLOAD_LIMIT]
    else:
        settings.maybePrint("Found: Videos")
        file_list = [random.choice(videos)]

    i = 1
    for file in sorted(file_list, key = lambda x: x['title']):
        print('Downloading: {} ({}/{})'.format(file['title'], i, folder_size))
        settings.maybePrint('filePath: '+tmp+"/"+str(file['title']))
        file.GetContentFile(tmp+"/"+str(file['title']))
        i+=1
    print('Downloaded: Performer')
    return [file_list, tmp, content_title]

# Download Scene
def download_scene(sceneFolder):
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Downloading Scene')
    tmp = settings.getTmp()
    global PYDRIVE
    content = None
    preview = None
    folder_list = PYDRIVE.ListFile({'q': "'"+sceneFolder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    for folder in folder_list:
        if folder['title'] == "content":
            content = folder
            settings.maybePrint("Content Folder: "+folder['id'])
        elif folder['title'] == "preview":
            preview = folder
            settings.maybePrint("Preview Folder: "+folder['id'])
    data = PYDRIVE.ListFile({'q': "'"+sceneFolder['id']+"' in parents and trashed=false and mimeType contains 'text/plain'"}).GetList()
    if len(data) == 0:
        print("Error: Missing Scene Data")
        return
    data = data[0]
    if data is None:
        print("Error: Missing Scene Data")
        return
    if content is None:
        print("Error: Missing Scene Content Folder")
        return
    if preview is None:
        print("Error: Missing Scene Preview Folder")
        return
    content = download_gallery(content)
    # move content to tmp/content
    tmp_content = os.path.join(content[1], "content")
    settings.maybePrint("Old Content Path: {}".format(content[1]))
    settings.maybePrint("New Content Path: {}".format(tmp_content))
    os.mkdir(tmp_content)
    for file in os.listdir(content[1]):
        file = os.path.join(content[1], file) 
        settings.maybePrint("Moving: {}".format(file))
        shutil.move(file, tmp_content)
    data.GetContentFile(os.path.join(tmp, "data.json"))
    # read data.json    
    with open(os.path.join(tmp, "data.json"), 'r', encoding='utf-8') as f:
        data = json.load(f)
    settings.maybePrint("data.json: {}".format(data))
    preview = PYDRIVE.ListFile({'q': "'"+preview['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\')"}).GetList()
    if len(preview) == 0:
        print("Error: Missing Scene Preview")
        return
    preview = preview[0]
    preview = download_file(preview)
    if "mp4" in preview:
        if str(settings.THUMBNAILING_PREVIEW) == "False":
            print("Error: Preview Thumbnailing Disabled")
            return
        preview = thumbnail_preview_fix(preview)
    if data is None:
        print("Error: Missing Scene Data")
        return
    if content is None:
        print("Error: Missing Scene Content")
        return
    if preview is None:
        print("Error: Missing Scene Preview")
        return
    print('Downloaded: Scene')
    return [tmp_content, preview, data, content]
    # return json.dumps({
    #     "content": content[0],
    #     "content_path": tmp_content,
    #     "preview": preview,
    #     "data": data
    # })

###############
##### Get #####
###############

# gets all the folders in the messages category
# gets all the images in the folders
# returns an array of json images w/ 
# [ 'image.folder' - 'image.title']
def get_images():
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Getting Images')
    global PYDRIVE
    OnlyFans_Images_Folder = get_folder_by_name("images")
    if OnlyFans_Images_Folder is None:
        print("Error: Unable to get Images Folder")
        return None
    random_folders = PYDRIVE.ListFile({'q': "'"+OnlyFans_Images_Folder+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    images_list = []
    for folder in random_folders:
        settings.maybePrint('checking folder: '+folder['title'])
        images_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\')"}).GetList()      
        for image_file in images_list_tmp:
            # images_list.append({"folder":folder['title'],"folder_id":folder['id'],"id":image_file['id'],"image":image_file['title']})
            images_list.append([folder, image_file])
    if len(images_list)==0:
        print('Warning: Missing Message Files')
        return
    return images_list
    # menu.selectImage -> [] -> menu.selectImage

def get_message_image(folderName):
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Getting Message Image: {}'.format(folderName))
    global PYDRIVE
    OnlyFans_Images_Folder = get_folder_by_name("messages")
    if OnlyFans_Images_Folder is None:
        print("Error: Unable to get Images Folder")
        return None
    random_folders = PYDRIVE.ListFile({'q': "'"+OnlyFans_Images_Folder+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    images_list = []
    random_image = None
    folder_name = None
    for folder in random_folders:
        if folder['title'] != str(folderName):
            continue
        if str(settings.VERBOSE) == "True":
            print('checking folder: '+folder['title'],end="")
        images_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\')"}).GetList()      
        if len(images_list_tmp)>0:
            images_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    if len(images_list)==0:
        print('Error: Missing Image File')
        return
    random_image = random.choice(images_list)
    folder_name = random_image['title'];
    print('Messages Folder: '+random_image['title'])
    random_image = PYDRIVE.ListFile({'q': "'"+random_image['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\')"}).GetList()
    random_image = random.choice(random_image)
    print('Messages Image: '+random_image['title'])
    return [random_image, folder_name]

# Downloads random image from Google Drive
def get_random_image():
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Getting Random Image')
    global PYDRIVE
    OnlyFans_Images_Folder = get_folder_by_name("images")
    if OnlyFans_Images_Folder is None:
        print("Error: Unable to get Images Folder")
        return None
    random_folders = PYDRIVE.ListFile({'q': "'"+OnlyFans_Images_Folder+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    images_list = []
    random_image = None
    folder_name = None
    for folder in random_folders:
        if str(settings.VERBOSE) == "True":
            print('checking folder: '+folder['title'],end="")
        images_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\')"}).GetList()      
        if len(images_list_tmp)>0:
            images_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    if len(images_list)==0:
        print('Error: Missing Image File')
        return
    random_image = random.choice(images_list)
    folder_name = random_image['title'];
    print('Random Folder: '+random_image['title'])
    random_image = PYDRIVE.ListFile({'q': "'"+random_image['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\')"}).GetList()
    random_image = random.choice(random_image)
    print('Random Image: '+random_image['title'])
    return [random_image, folder_name]

# Downloads random gallery from Google Drive
def get_random_gallery():
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Getting Random Gallery')
    global PYDRIVE
    OnlyFans_Galleries_Folder = get_folder_by_name("galleries")
    if OnlyFans_Galleries_Folder is None:
        print("Error: Unable to get Galleries Folder")
        return None
    random_folders = PYDRIVE.ListFile({'q': "'"+OnlyFans_Galleries_Folder+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    folder_list = []
    random_gallery = None
    folder_name = None
    for folder in random_folders:
        if str(settings.VERBOSE) == "True":
            print('checking galleries: {}'.format(folder['title']),end="")
        gallery_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
        if len(gallery_list_tmp)>0:
            folder_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    random.shuffle(folder_list)
    for folder in folder_list:
        if str(settings.VERBOSE) == "True":
            print('checking gallery: {}'.format(folder['title']),end="")
        gallery_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
        random_gallery_tmp = random.choice(gallery_list_tmp)
        gallery_list_tmp_tmp = PYDRIVE.ListFile({'q': "'"+random_gallery_tmp['id']+"' in parents and trashed=false and (mimeType contains \'image/jpeg\' or mimeType contains \'image/jpg\' or mimeType contains \'image/png\')"}).GetList()
        if len(gallery_list_tmp_tmp)>0:
            folder_name = folder['title']
            random_gallery = random_gallery_tmp
            settings.maybePrint(" -> found")
        else:
            settings.maybePrint(" -> empty")
    if not random_gallery:
        print('Error: Missing Gallery Folder')
        return
    print('Random Gallery: '+random_gallery['title'])
    return [random_gallery, folder_name]

# Downloads random performer from Google Drive
def get_random_performer():
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Getting Random Performer')
    global PYDRIVE
    OnlyFans_Performers_Folder = get_folder_by_name("performers")
    if OnlyFans_Performers_Folder is None:
        print("Error: Unable to get Performers Folder")
        return None
    random_folders = PYDRIVE.ListFile({'q': "'"+OnlyFans_Performers_Folder+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    performer_list = []
    random_performer = None
    # print('random folders: '+str(random_folders))
    for folder in random_folders:
        random_folder_folder = random.choice(random_folders)
        settings.maybePrint('random performer: '+random_folder_folder['title'])
        performer_content_list = PYDRIVE.ListFile({'q': "'"+random_folder_folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
        # print('random folders: '+str(performer_list))
        if len(performer_content_list)==0:
            settings.maybePrint('- skipping empty performer: '+random_folder_folder['title'])
        elif len(performer_content_list)>0:
            settings.maybePrint('- performer found: '+random_folder_folder['title'])
            performer_list.append(random_folder_folder)
    if len(performer_list)==0:
        print('Error: Missing Performer Folder')
        return
    random_performer = random.choice(performer_list)
    print('Random Performer: '+random_performer['title'])
    return random_performer

# Downloads random video from Google Drive
def get_random_video():
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Getting Random Video')
    global PYDRIVE
    OnlyFans_Videos_Folder = get_folder_by_name("videos")
    if OnlyFans_Videos_Folder is None:
        print("Error: Unable to get Videos Folder")
        return None
    random_folders = PYDRIVE.ListFile({'q': "'"+OnlyFans_Videos_Folder+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    video_list = []
    random_video = None
    folder_name = None
    for folder in random_folders:
        if str(settings.VERBOSE) == "True":
            print('checking folder: '+folder['title'],end="")
        video_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'video/mp4'"}).GetList()
        if len(video_list_tmp)>0:
            video_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    if len(video_list)==0:
        print('Error: Missing Video File')
        return
    random_video = random.choice(video_list)
    folder_name = random_video['title'];
    print('Random Folder: '+random_video['title'])
    random_video = PYDRIVE.ListFile({'q': "'"+random_video['id']+"' in parents and trashed=false and mimeType contains 'video/mp4'"}).GetList()
    random_video = random.choice(random_video)
    print('Random Video: '+random_video['title'])
    return [random_video, folder_name]

# Downloads random scene from Google Drive
def get_random_scene():
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    print('Getting Random Scene')
    global PYDRIVE
    OnlyFans_Scenes_Folder = get_folder_by_name("scenes")
    if OnlyFans_Scenes_Folder is None:
        print("Error: Unable to get Scenes Folder")
        return None
    random_folders = PYDRIVE.ListFile({'q': "'"+OnlyFans_Scenes_Folder+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    folder_list = []
    random_scene = None
    folder_name = None
    for folder in random_folders:
        if str(settings.VERBOSE) == "True":
            print('checking scenes: '+folder['title'],end="")
        scene_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
        if len(scene_list_tmp)>0:
            folder_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    random.shuffle(folder_list)
    for folder in folder_list:
        if str(settings.VERBOSE) == "True":
            print('checking scene: '+folder['title'],end="")
        scene_list_tmp_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'text/plain'"}).GetList()
        if len(scene_list_tmp_tmp)>0:
            folder_name = folder['title']
            random_scene = folder
            settings.maybePrint(" -> found")
        else:
            settings.maybePrint(" -> empty")
    if not random_scene:
        print('Error: Missing Scene Folders')
        return
    print('Random Scene: '+random_scene['title'])
    return [random_scene, folder_name]

##################
##### Upload #####
##################

def upload_file(filename=None, mimetype="video/mp4"):
    global AUTH
    if not AUTH:
        AUTH = authGoogle()
    if filename == None:
        print("Error: Missing Filename")
        return
    file_metadata = {
        'name': str(filename),
        'mimeType': str(mimetype)
    }
    media = MediaFileUpload(str(filename), mimetype=str(mimetype), resumable=True)
    file = DRIVE.files().create(body=file_metadata, media_body=media, fields='id').execute()
    print('File ID: %s' % file.get('id'))

##################
##### FFMPEG #####
##################

def repair(path):
    if (settings.SKIP_REPAIR):
        print("Warning: Skipping Repair")
        return path
    repairedPath = str(path).replace(".mp4", "_fixed.mp4")
    try:
        print("Repairing: {} <-> {}".format(path, settings.WORKING_VIDEO))
        if str(settings.DEBUG) == "True":
            fixed = subprocess.call(['untrunc', str(settings.WORKING_VIDEO), str(path)])
        else:
            subprocess.Popen(['untrunc', str(settings.WORKING_VIDEO), str(path)],stdin=FNULL,stdout=FNULL)
        fixed.communicate()
    except AttributeError:
        if os.path.isfile(str(path)+"_fixed.mp4"):
            shutil.move(str(path)+"_fixed.mp4", repairedPath)
            print("Repair Complete")
    except:
        settings.maybePrint(sys.exc_info()[0])
        print("Warning: Skipping Repair")
        return path
    print("Repair Successful")
    return str(repairedPath)

def reduce(path):
    if (settings.SKIP_REDUCE):
        print("Warning: Skipping Reduction")
        return path
    reducedPath = str(path).replace(".mp4", "_reduced.mp4")
    try:
        settings.maybePrint("Reducing: {}".format(path))
        try:
            clip = VideoFileClip(str(path))
            settings.maybePrint("Length: {}".format(clip.duration))
            bitrate = 1000000000 / int(clip.duration)
            settings.maybePrint("Bitrate: {}".format(bitrate))
        except FileNotFoundError:
            print("Error: Missing File to Reduce")
            return path
        loglevel = "quiet"
        if str(settings.DEBUG) == "True":
            loglevel = "debug"
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-err_detect', 'ignore_err', '-y', '-i', str(path), '-c', 'copy', '-c:v', 'libx264', '-c:a', 'aac', '-strict', '2', '-crf', '26', '-b:v', str(bitrate), str(reducedPath)])
        p.communicate()
    except FileNotFoundError:
        print("Warning: Ignoring Fixed Video")
        return reduce(str(path).replace(".mp4", "_fixed.mp4"))
    except Exception as e:
        settings.maybePrint(e)
        if "Conversion failed!" in str(e):
            print("Error: Conversion Failure")
            return path                    
    print("Reduction Complete")
    originalSize = os.path.getsize(str(path))
    newSize = os.path.getsize(str(reducedPath))
    print("Original Size: {}kb - {}mb".format(originalSize/1000, originalSize/1000000))
    print("Reduced Size: {}kb - {}mb".format(newSize/1000, newSize/1000000))
    if int(originalSize) < int(newSize):
        print("Warning: Original Size Smaller")
        return path
    if int(newSize) == 0:
        print("Error: Missing Reduced File")
        return path
    return reducedPath

def thumbnail_fix(path):
    if (settings.SKIP_THUMBNAIL):
        print("Warning: Skipping Thumbnail")
        return path
    thumbedPath = str(path).replace(".mp4", "_thumbed.mp4")
    try:
        print("Thumbnailing: {}".format(path))
        loglevel = "quiet"
        if str(settings.DEBUG) == "True":
            loglevel = "debug"
        thumbnail_path = os.path.join(os.path.dirname(str(path)), 'thumbnail.png')
        settings.maybePrint("thumbnail path: {}".format(thumbnail_path))
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-i', str(path),'-ss', '00:00:00.000', '-vframes', '1', str(thumbnail_path)])
        p.communicate()
    except FileNotFoundError:
        print("Warning: Ignoring Thumbnail")
        return path
    except AttributeError:
        print("Thumbnailing: Captured PNG")
    except:
        settings.maybePrint(sys.exc_info()[0])
        print("Error: Thumbnailing Fuckup")
        return path
    try:
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-y', '-i', str(path), '-i', str(thumbnail_path), '-pix_fmt', 'rgb24', '-c', 'copy', '-acodec', 'copy', '-vcodec', 'copy', '-map', '0', '-map', '1', '-c:v:1', 'png', '-disposition:v:1', 'attached_pic', str(thumbedPath)])
        p.communicate()
    except AttributeError:
        print("Thumbnailing: Added PNG")
    except:
        settings.maybePrint(sys.exc_info()[0])
        print("Error: Thumbnailing Adding Fuckup")
        return path
    print("Thumbnailing Complete")
    originalSize = os.path.getsize(str(path))
    newSize = os.path.getsize(str(thumbedPath))
    print("Original Size: {}kb - {}mb".format(originalSize/1000, originalSize/1000000))
    print("Thumbed Size: {}kb - {}mb".format(newSize/1000, newSize/1000000))
    if int(originalSize) < int(newSize):
        print("Warning: Original Size Smaller")
        return path
    if int(newSize) == 0:
        print("Error: Missing Thumbnailed File")
        return path
    return thumbedPath

def thumbnail_preview_fix(path):
    # if (settings.SKIP_THUMBNAIL):
        # print("Warning: Skipping Thumbnail")
        # return
    try:
        print("Thumbnailing: {}".format(path))
        loglevel = "quiet"
        if str(settings.DEBUG) == "True":
            loglevel = "debug"
        thumbnail_path = os.path.join(os.path.dirname(str(path)), 'thumbnail.png')
        settings.maybePrint("thumbnail path: {}".format(thumbnail_path))
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-i', str(path),'-ss', '00:00:00.000', '-vframes', '1', str(thumbnail_path)])
        p.communicate()
        print("Thumbnailing Complete")
        return thumbedPath
    except FileNotFoundError:
        print("Warning: Ignoring Thumbnail")
    except AttributeError:
        print("Thumbnailing: Captured PNG")
    except:
        settings.maybePrint(sys.exc_info()[0])
        print("Error: Thumbnailing Fuckup")    


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="OnlySnarf",
    version="1.4.2",
    author="Skeetzo",
    author_email="WebmasterSkeetzo@gmail.com",
    url = 'https://github.com/skeetzo/onlysnarf',
    keywords = ['OnlyFans', 'Content', 'OnlySnarf'],
    description="OnlyFans Content Distribution Tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    # package_data={'': ['/path/to/google']},
    # include_package_data=True,
    install_requires=[
        'selenium',
        'pydrive',
        'pathlib', 
        'chromedriver-binary==77.0.3865.40',
        'moviepy',
        'google-api-python-client',
        'httplib2',
        'python-crontab'
        ],
    entry_points={
        'console_scripts' : [
            'onlysnarf = OnlySnarf.menu:main_other',
            'onlysnarf-config = OnlySnarf.config:main'
        ]
    },
    classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Topic :: System :: Shells',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.4',
    "Operating System :: OS Independent",
  ]
)